

class TaskStatus:
    FAILURE = 'FAILURE'
    REJECTED = 'REJECTED'
    UNKNOWN = 'UNKNOWN'

    CHOICES = (
        (FAILURE, 'Failure'),
        (REJECTED, 'Rejected'),
        (UNKNOWN, 'Unknown'),
    )
